package com.persagy.apm.energy.report.monthly.config.type.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.common.model.dto.Sort;
import com.persagy.apm.common.model.vo.EnumVO;
import com.persagy.apm.energy.report.monthly.config.type.constant.enums.BelongTypeEnum;
import com.persagy.apm.energy.report.monthly.config.type.dao.ReportTypeMapper;
import com.persagy.apm.energy.report.monthly.config.type.model.ConvertReportTypeTool;
import com.persagy.apm.energy.report.monthly.config.type.model.ReportType;
import com.persagy.apm.energy.report.monthly.config.type.model.dto.AddReportTypeDTO;
import com.persagy.apm.energy.report.monthly.config.type.model.dto.PageQueryReportTypeDTO;
import com.persagy.apm.energy.report.monthly.config.type.model.dto.QueryReportTypeDTO;
import com.persagy.apm.energy.report.monthly.config.type.model.dto.UpdateReportTypeDTO;
import com.persagy.apm.energy.report.monthly.config.type.model.vo.ResponseReportTypeListItemVO;
import com.persagy.apm.energy.report.monthly.config.type.service.IReportTypeEnumService;
import com.persagy.apm.energy.report.monthly.config.type.service.IReportTypeService;
import com.persagy.apm.energy.report.saasweb.model.vo.PartitionVO;
import com.persagy.apm.energy.report.saasweb.model.vo.SimpleProjectVO;
import com.persagy.apm.energy.report.saasweb.service.ISaasWebService;
import org.apache.commons.lang.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 报告类型(ReportType) service层
 *
 * @author lixing
 * @version V1.0 2021-05-19 10:39:15
 */
@Service
public class ReportTypeServiceImpl extends ServiceImpl<ReportTypeMapper, ReportType>
        implements IReportTypeService {
    @Autowired
    ISaasWebService saasWebService;
    @Autowired
    IReportTypeEnumService reportTypeEnumService;

    /**
     * 创建报告类型
     *
     * @return 报告类型主键
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    @Override
    public String createReportType(AddReportTypeDTO addReportTypeDTO) {
        ReportType reportType = ConvertReportTypeTool.INSTANCE.convertAddDto2Entity(addReportTypeDTO);
        // 设置默认值
        setDefaultValue(reportType);
        save(reportType);
        return reportType.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param reportType 报告类型实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ReportType reportType) {
        reportType.setCreator(PoemsContext.getContext().getUserId());
        

    }

    /**
     * 报告类型详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    @Override
    public ReportType queryReportTypeDetail(String id) {
        ReportType reportType = getById(id);
        if (reportType == null) {
            throw new IllegalArgumentException("查看ReportType详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return reportType;
    }

    /**
     * 更新报告类型
     *
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    @Override
    public void updateReportType(UpdateReportTypeDTO updateReportTypeDTO) {
        ReportType reportType = getById(updateReportTypeDTO.getId());
        reportType = ConvertReportTypeTool.INSTANCE.convertUpdateDto2Entity(reportType, updateReportTypeDTO);
        reportType.setModifier(PoemsContext.getContext().getUserId());
        updateById(reportType);
    }

    /**
     * 校验报告类型是否可删除
     *
     * @param id 报告类型主键
     * @return 报告类型do类
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    public ReportType checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ReportType时发生异常，主键为空");
        }

        ReportType reportType = getById(id);

        if (reportType == null) {
            throw new IllegalArgumentException("删除ReportType时发生异常，找不到要删除的数据，id:" + id);
        }

        return reportType;
    }

    /**
     * 删除报告类型
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    @Override
    public void deleteReportType(String id) {
        // 校验是否可删除
        ReportType reportType = checkDeletable(id);

        reportType.setValid(ValidEnum.FALSE.getType());
        updateById(reportType);
    }

    /**
     * 查询报告类型
     *
     * @return List<ReportType>
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    @Override
    public List<ReportType> queryReportTypeList(QueryReportTypeDTO queryReportTypeDTO) {
        QueryWrapper<ReportType> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportType.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ReportType.PROP_CREATIONTIME);

        if (queryReportTypeDTO != null) {
            if (StringUtils.isNotEmpty(queryReportTypeDTO.getName())) {
                queryWrapper.like(ReportType.PROP_NAME, queryReportTypeDTO.getName());
            }

            if (queryReportTypeDTO.getBusinessType() != null) {
                queryWrapper.eq(ReportType.PROP_BUILDING_TYPE, queryReportTypeDTO.getBusinessType());
            }

            if (queryReportTypeDTO.getBelongType() != null) {
                queryWrapper.eq(ReportType.PROP_BELONG_TYPE, queryReportTypeDTO.getBelongType());
            }
        }

        return list(queryWrapper);
    }

    /**
     * 分页查询报告类型
     *
     * @return IPage<ReportType>
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    @Override
    public IPage<ReportType> pageQueryReportType(PageQueryReportTypeDTO pageQueryReportTypeDTO) {
        QueryWrapper<ReportType> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportType.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ReportType> pageParam = new Page<>(pageQueryReportTypeDTO.getPage(), pageQueryReportTypeDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryReportTypeDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ReportType.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryReportTypeDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        
        if (StringUtils.isNotEmpty(pageQueryReportTypeDTO.getName())) {
            queryWrapper.like(ReportType.PROP_NAME, pageQueryReportTypeDTO.getName());
        }

        if (pageQueryReportTypeDTO.getBusinessType() != null) {
            queryWrapper.eq(ReportType.PROP_BUILDING_TYPE, pageQueryReportTypeDTO.getBusinessType());
        }

        if (pageQueryReportTypeDTO.getBelongType() != null) {
            queryWrapper.eq(ReportType.PROP_BELONG_TYPE, pageQueryReportTypeDTO.getBelongType());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public IPage<ResponseReportTypeListItemVO> queryAvailableReportTypes() {
        IPage<ResponseReportTypeListItemVO> result = new Page<>();
        List<ResponseReportTypeListItemVO> listItemVOList = Lists.newArrayList();

        List<EnumVO> buildingTypes = reportTypeEnumService.queryBuildingTypeEnum();
        for (EnumVO buildingType : buildingTypes) {
            List<SimpleProjectVO> availableProjects = saasWebService.getAvailableProjects(buildingType.getKey());
            if (!CollectionUtils.isEmpty(availableProjects)) {
                List<String> availableBelongIds = availableProjects.stream().map(
                        SimpleProjectVO::getProjectId).collect(Collectors.toList());
                QueryReportTypeDTO queryReportTypeDTO = new QueryReportTypeDTO();
                queryReportTypeDTO.setBelongType(BelongTypeEnum.PROJECT.getType());
                queryReportTypeDTO.setBusinessType(buildingType.getKey());
                List<ReportType> reportTypes = queryReportTypeList(queryReportTypeDTO);
                List<ResponseReportTypeListItemVO> voList = packageReportTypeListItemVOList(
                        reportTypes, availableBelongIds);
                listItemVOList.addAll(voList);
            }
            List<String> availableAreaIds = saasWebService.getAvailableAreaIds(buildingType.getKey());
            if (!CollectionUtils.isEmpty(availableAreaIds)) {
                QueryReportTypeDTO queryReportTypeDTO = new QueryReportTypeDTO();
                queryReportTypeDTO.setBelongType(BelongTypeEnum.AREA.getType());
                queryReportTypeDTO.setBusinessType(buildingType.getKey());
                List<ReportType> reportTypes = queryReportTypeList(queryReportTypeDTO);
                List<ResponseReportTypeListItemVO> voList = packageReportTypeListItemVOList(
                        reportTypes, availableAreaIds);
                listItemVOList.addAll(voList);
            }
        }

        result.setRecords(listItemVOList);
        result.setTotal(listItemVOList.size());

        return result;
    }

    /**
     * 包装reportTypeListItemVO列表
     *
     * @param reportTypes 报告类型对象列表
     * @param availableBelongIds 报告类型下可用的报告所属
     * @return 包装后的reportTypeListItemVO列表
     * @author lixing
     * @version V1.0 2021/6/23 5:39 下午
     */
    private List<ResponseReportTypeListItemVO> packageReportTypeListItemVOList(
            List<ReportType> reportTypes, List<String> availableBelongIds) {
        if (CollectionUtils.isEmpty(reportTypes)) {
            return Lists.newArrayList();
        }
        List<ResponseReportTypeListItemVO> results = new ArrayList<>();
        for (ReportType reportType : reportTypes) {
            ResponseReportTypeListItemVO responseReportTypeListItemVO = ConvertReportTypeTool.INSTANCE.
                    convert2ResponseListItemDTO(reportType);
            responseReportTypeListItemVO.setAvailableBelongIds(availableBelongIds);
            results.add(responseReportTypeListItemVO);
        }
        return results;
    }

}
